/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.gogrid;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.dc.DataCenter;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.dc.Region;
import org.jclouds.gogrid.domain.Option;

public class GoGridDc implements DataCenterServices {
    private GoGrid provider = null;
    
    GoGridDc(GoGrid provider) { 
        this.provider = provider;
    }
    
    @Override
    public DataCenter getDataCenter(String dataCenterId) throws InternalException, CloudException {
        for( DataCenter dc : listDataCenters(provider.getContext().getRegionId()) ) {
            if( dc.getProviderDataCenterId().equals(dataCenterId) ) {
                return dc;
            }
        }
        return null;
    }

    @Override
    public String getProviderTermForDataCenter(Locale locale) {
        return "data center";
    }

    @Override
    public String getProviderTermForRegion(Locale locale) {
        return "region";
    }

    @Override
    public Region getRegion(String regionId) throws InternalException, CloudException {
        for( Region region : listRegions() ) {
            if( region.getProviderRegionId().equals(regionId) ) {
                return region;
            }
        }
        return null;
    }

    @Override
    public Collection<DataCenter> listDataCenters(String regionId) throws InternalException, CloudException {
        Region region = getRegion(regionId);
        
        if( region == null ) {
            return null;
        }
        DataCenter dc = new DataCenter();
        dc.setActive(true);
        dc.setAvailable(true);
        dc.setName(region.getName() + "a");
        dc.setProviderDataCenterId(region.getProviderRegionId() + "a");
        dc.setRegionId(regionId);
        return Collections.singletonList(dc);
    }

    static private List<Region> regions = null;
    
    @Override
    public Collection<Region> listRegions() throws InternalException, CloudException {
        if( regions == null ) {
            ArrayList<Region> list = new ArrayList<Region>();
        
            for( Option option : provider.getCloudClient().getApi().getServerServices().getDatacenters() ) {
                Region region = new Region();
                
                region.setProviderRegionId(String.valueOf(option.getId()));
                region.setActive(true);
                region.setAvailable(true);
                region.setName(option.getName());
                if( region.getName() == null ) {
                    region.setName(region.getProviderRegionId());
                }
                list.add(region);
            }
            regions = Collections.unmodifiableList(list);
        }
        return regions;
    }
}
